#include <PalmOS.h>

#include "Plua.h"
#include "PluaAPI.h"

#define LIBNAME "3dlib"
#define BUTTON_LABEL 32

typedef struct {
  UInt32 text, light, normal, dark, blank;
  FontID font;
  Boolean pressed;
  char label[BUTTON_LABEL];
} Button3D;

static void drawbutton(lua_State *L, RectangleType *rect, Button3D *b3d)
{
  PLUA_HEADER;
  int width, height, d;
  Int16 x, y, dx, dy;
  FontID old;

  d = b3d->pressed ? 1 : 0;
  old = FntSetFont(b3d->font);
  plua_ptextdimensions(b3d->label, &width, &height);

  x = rect->topLeft.x;
  y = rect->topLeft.y;
  dx = rect->extent.x;
  dy = rect->extent.y;
  plua_pcolor(b3d->normal, b3d->normal);
  plua_pbox(x, y, dx,dy); 
  plua_pcolor(b3d->pressed ? b3d->dark : b3d->light, b3d->normal);
  plua_pline(x, y, x, y+dy-1);
  plua_pline(x, y, x+dx-1, y); 
  plua_pcolor(b3d->pressed ? b3d->light : b3d->dark, b3d->normal);
  plua_pline(x+dx-1, y, x+dx-1, y+dy-1);
  plua_pline(x, y+dy-1, x+dx-1, y+dy-1);

  plua_pcolor(b3d->text, b3d->normal);
  plua_pmoveto(rect->topLeft.x + (rect->extent.x - width) / 2 + d,
               rect->topLeft.y + (rect->extent.y - height) / 2 + d);
  plua_print(b3d->label);
  FntSetFont(old);
}

Boolean pbutton3d_callback(lua_State *L, PluaGadgetEvent event, int id,
                           void *data, void *arg, RectangleType *rect)
{
  PLUA_HEADER;
  Button3D *b3d;
  EventType evt;
  Int16 *state;

  b3d = (Button3D *)data;
  
  switch (event) {
    case gadgetDraw:
      drawbutton(L, rect, b3d);
      break;
    case gadgetUp:
      b3d->pressed = false;
      drawbutton(L, rect, b3d);
      MemSet(&evt, sizeof(evt), 0);
      evt.eType = ctlSelectEvent;
      evt.data.ctlSelect.controlID = id;
      evt.data.ctlSelect.on = 1;
      evt.data.ctlSelect.value = 1;
      EvtAddEventToQueue(&evt);
      break;
    case gadgetErase:
      plua_pcolor(b3d->blank, b3d->blank);
      plua_pbox(rect->topLeft.x, rect->topLeft.y,
                rect->extent.x, rect->extent.y);
      break;
    case gadgetDown:
      b3d->pressed = true;
      drawbutton(L, rect, b3d);
      break;
    case gadgetDelete:
      free(b3d);
      break;
    case gadgetGetState:
      state = (Int16 *)arg;
      *state = b3d->pressed ? 1 : 0;
      break;
    case gadgetSetState:
      state = (Int16 *)arg;
      b3d->pressed = state ? true : false;
      break;
    case gadgetGetText:
      if (arg)
        StrNCopy((char *)arg, b3d->label, BUTTON_LABEL-1);
      break;
    case gadgetSetText:
      if (arg)
        StrNCopy(b3d->label, (char *)arg, BUTTON_LABEL-1);
  }

  return true;
}

int pbutton3d(lua_State *L)
{
  PLUA_HEADER;
  Button3D *b3d;
  char *label;
  Int16 id, width, height;
  long fg, bg;
  int r, g, b;

  if ((b3d = malloc(sizeof(Button3D))) == NULL)
    return 0;

  label = (char *)luaL_check_string(L, 1);
  width = luaL_check_int(L, 2);
  height = luaL_check_int(L, 3);

  StrNCopy(b3d->label, label, BUTTON_LABEL-1);

  plua_getcolor(&fg, &bg);
  plua_getrgb(bg, &r, &g, &b);
  b3d->normal = plua_prgb(r, g, b);
  b3d->light = plua_prgb(1.50*r, 1.50*g, 1.50*b);
  b3d->dark = plua_prgb(0.50*r, 0.50*g, 0.50*b);
  b3d->blank = plua_prgb(255, 255, 255);

  plua_getrgb(fg, &r, &g, &b);
  b3d->text = plua_prgb(r, g, b);
  b3d->font = FntGetFont();
  b3d->pressed = false;

  if ((id = plua_creategadget(L, width, height, b3d, pbutton3d_callback))==-1) {
    free(b3d);
    return 0;
  }

  lua_pushnumber(L, id);
  return 1;
}

// This function is called when the library is first loaded by a Plua program.
// You may alloc and register your gloabal data, and also register Lua
// functions with lua_register().

Err PluaLibInit(lua_State *L)
{
  PLUA_HEADER;
  const struct luaL_reg lib3d[] = {
    {"pbutton3d", pbutton3d},
    {NULL, NULL}
  };

  // register C functions in Lua
  luaL_openl(L, lib3d);

  // return sucess
  return 0;
}

// This function is called when the library is explicitely removed by a Plua
// program or when Plua exits, whichever happens first. You MUST unregister
// all the functions defined in PluaLibInit() and also free the global data.
// If you do not unregister a function and a Plua program calls it after
// the library is removed PalmOS will CRASH.

Err PluaLibFinish(lua_State *L)
{
  PLUA_HEADER;

  // unregister C functions in Lua
  lua_pushnil(L);
  lua_setglobal(L, "pbutton3d");

  // return sucess
  return 0;
}
