-- LuaCalc.lua
-- Copyright (c) 2004 Marcio M. Andrade

-- This program requires the library 3dlib.prc, provided inside libkit

-- Stack definition

stack = { index = 0 }

function stack:push(v)
  self[self.index] = v
  self.index = self.index + 1
end

function stack:pop()
  if self.index > 0 then
    self.index = self.index - 1
    return self[self.index]
  end
  return 0
end

-- Display definition

display = {}

function display:init(x,y,w,h,fg,bg)
  self.value = 0
  self.max = 15
  self.x = x
  self.y = y
  self.w = w
  self.h = h
  self.fg = fg
  self.bg = bg
  self.decimal = nil
  self.frozen = nil
  self.mode = ""
end

function display:update()
  pbox(self.x,self.y,self.w,self.h,self.bg)
  pcolor(self.fg,self.bg)
  local w,h = pfont(0)
  local v = tostring(self.value)
  local e = strfind(v,"e")

  if e then
    v = strsub(v,1,min(e-1,10)) .. strsub(v,e)
  elseif v == "[inf]" or v == "[nan]" then
    v,self.value = 0,0
    pmoveto(self.x+2, self.y + self.h - h)
    write("ERR")
  else
    if strlen(v) > self.max then
      v = strsub(v,1,self.max)
    end
  end

  _,h = pfont(7)
  w = ptextwidth(v)
  pmoveto(self.x + self.w - w, self.y + (self.h - h) / 2)
  write(v)

  pfont(0)
  pmoveto(self.x+2, self.y)
  write(self.mode)
end

function display:setmode(m)
  self.mode = m
  self:update()
end

function display:reset()
  self.value = 0
  self.decimal = nil
  self.frozen = nil
  self:update()
end

function display:set(v)
  self.value = v
  self.decimal = nil
  self.frozen = 1
  self:update()
end

function display:get()
  if strsub(self.value,-1) == "." then
    self.value = strsub(self.value,1,-2)
  end
  return tonumber(self.value)
end

function display:isfrozen()
  return self.frozen
end

function display:insert(d)
  if self.frozen then
    if d == "." then
      self.value = "0."
      self.decimal = 1
    else
      self.value = d
      self.decimal = nil
    end
    self.frozen = nil
  else
    if self.value == 0 then
      if d == "." then
        self.value = "0."
        self.decimal = 1
      elseif d > 0 then
        self.value = d
      end
    else
      if d == "." then
        if not self.decimal and strlen(self.value) < self.max then
          self.value = self.value .. d
          self.decimal = 1
        end
      elseif strlen(self.value) < self.max then
        self.value = self.value .. d
      end
    end
  end
  self:update()
end

function display:delete()
  if self.frozen then
    self.value = 0
    self.decimal = nil
    self.frozen = nil
  else
    local i = strlen(self.value)
    if (i > 1) then
      if self.decimal then
        local d = strsub(self.value,i,i)
        if d == "." then
          self.decimal = nil
        end
      end
      self.value = strsub(self.value,1,i-1)
    else
      self.value = "0"
      self.decimal = nil
    end
  end
  self:update()
end

-- Calculator operators

function fadd()
  display:set(stack:pop() + display:get())
end

function fsub()
  display:set(stack:pop() - display:get())
end

function fmult()
  display:set(stack:pop() * display:get())
end

function fdiv()
  display:set(stack:pop() / display:get())
end

function fcos()
  local r = display:get()
  if mode == "RAD" then
    r = deg(r)
  end
  display:set(cos(r))
end

function facos()
  local r = acos(display:get())
  if mode == "RAD" then
    r = rad(r)
  end
  display:set(r)
end

function fsin()
  local r = display:get()
  if mode == "RAD" then
    r = deg(r)
  end
  display:set(sin(r))
end

function fasin()
  local r = asin(display:get())
  if mode == "RAD" then
    r = rad(r)
  end
  display:set(r)
end

function ftan()
  local r = display:get()
  if mode == "RAD" then
    r = deg(r)
  end
  display:set(tan(r))
end

function fatan()
  local r = atan(display:get())
  if mode == "RAD" then
    r = rad(r)
  end
  display:set(r)
end

function fatan2()
  local r = atan2(stack:pop(), display:get())
  if mode == "RAD" then
    r = rad(r)
  end
  display:set(r)
end

function fmode()
  if mode == "RAD" then
    mode = "DEG"
  else
    mode = "RAD"
  end
  display:setmode(mode)
end

function finv()
  display:set(1 / display:get())
end

function fln()
  display:set(log(display:get()))
end

function fsqrt()
  display:set(sqrt(display:get()))
end

function fpow()
  display:set(pow(stack:pop(), display:get()))
end

function fexp()
  display:set(exp(display:get()))
end

function frand()
  stack:push(display:get())
  display:set(random())
end

function fsign()
  if display:get() ~= 0 then
    display:set(-display:get())
  end
end

function back()
  display:delete()
end

function enter()
  stack:push(display:get())
  display:reset()
end

function clear()
  display:reset()
end

function sto()
  local index,value = display:get(),stack:pop()
  display:set(value)
  memory[index] = value
end

function rcl()
  local index = display:get()
  local value = memory[index]
  if not value then
    value = 0
  end
  display:set(value)
end

function init()
  l3d = loadlib("3dlib")
  if not l3d then
    palert("This app requires 3dlib.prc")
    exit()
  end
  randomseed(date("%H%M%S"))
  local black = prgb(0,0,0)
  white = prgb(255,255,255)
  pcolor(black,prgb(192,192,192))
  pclear()
  local width,height = pmode()
  local displayHeight = (3 * height) / 20
  mode = "RAD"
  display:init(2,2,width-4,displayHeight-4,black,prgb(96,96,0))
  display:setmode(mode)
  local keyboardHeight = height - displayHeight
  local rows,columns = 6,6
  local bHeight,bWidth = keyboardHeight / rows,width / columns
  local x,y = 0,displayHeight

  f = {}
  memory = {}
  orange = prgb(255,128,0)
  green  = prgb(0,128,0)
  blue   = prgb(0,0,255)
  red    = prgb(255,0,0)
  pfont(0)

  pcolor(white,green)
  x,y = 0,displayHeight
  pmoveto(x+2,y+2) f[pbutton3d("ln",   bWidth-4,bHeight-4)] = fln   y = y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("exp",  bWidth-4,bHeight-4)] = fexp  y = y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("1/x",  bWidth-4,bHeight-4)] = finv  y = y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("sqrt", bWidth-4,bHeight-4)] = fsqrt y = y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("y^x",  bWidth-4,bHeight-4)] = fpow  y = y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("rnd",  bWidth-4,bHeight-4)] = frand

  pcolor(white,orange)
  x,y = bWidth,displayHeight
  pmoveto(x+2,y+2) f[pbutton3d("cos", bWidth-4,bHeight-4)] = fcos  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("sin", bWidth-4,bHeight-4)] = fsin  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("tan", bWidth-4,bHeight-4)] = ftan  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("mod", bWidth-4,bHeight-4)] = fmode x = x + bWidth
  pcolor(white,red)
  pmoveto(x+2,y+2) f[pbutton3d("sto", bWidth-4,bHeight-4)] = sto
  pcolor(white,orange)
  x,y = bWidth,y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d("acos", bWidth-4,bHeight-4)] = facos  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("asin", bWidth-4,bHeight-4)] = fasin  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("atan", bWidth-4,bHeight-4)] = fatan  x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("atn2", bWidth-4,bHeight-4)] = fatan2 x = x + bWidth
  pcolor(white,red)
  pmoveto(x+2,y+2) f[pbutton3d("rcl",  bWidth-4,bHeight-4)] = rcl

  pcolor(white,blue)
  x,y = bWidth,y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d(7,   bWidth-4,bHeight-4)] = 7    x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(8,   bWidth-4,bHeight-4)] = 8    x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(9,   bWidth-4,bHeight-4)] = 9    x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("/", bWidth-4,bHeight-4)] = fdiv x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("<", bWidth-4,bHeight-4)] = back

  x,y = bWidth,y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d(4,   bWidth-4,bHeight-4)] = 4     x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(5,   bWidth-4,bHeight-4)] = 5     x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(6,   bWidth-4,bHeight-4)] = 6     x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("*", bWidth-4,bHeight-4)] = fmult x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("C", bWidth-4,bHeight-4)] = clear

  x,y = bWidth,y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d(1,     bWidth-4,bHeight-4)] = 1       x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(2,     bWidth-4,bHeight-4)] = 2       x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(3,     bWidth-4,bHeight-4)] = 3       x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("-",   bWidth-4,bHeight-4)] = fsub    x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("Ent", bWidth-4,bHeight*2-4)] = enter

  x,y = bWidth,y + bHeight
  pmoveto(x+2,y+2) f[pbutton3d(0,     bWidth-4,bHeight-4)] = 0     x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("+/-", bWidth-4,bHeight-4)] = fsign x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d(".",   bWidth-4,bHeight-4)] = "."   x = x + bWidth
  pmoveto(x+2,y+2) f[pbutton3d("+",   bWidth-4,bHeight-4)] = fadd

  display:reset()
end

function eventloop()
  while 1 do
    local e,id = pevent()
    if e == ctlSelect then
      if type(f[id]) == "function" then
        f[id]()
      else
        if display:isfrozen() then
          stack:push(display:get())
        end
        display:insert(f[id])
      end
    end
  end
end

init()
eventloop()
